/// <reference types="node" />
import { RequestOptions } from 'http';
import { Har } from 'har-format';
import { BandwidthLimitsResponse, BlacklistItem, NewHarPageArgs, ProxyList, SetBandwidthLimitArgs, SetBlacklistArgs, SetRetryCountArgs, SetRewriteArgs, SetTimeoutsArgs, SetWhitelistArgs, StartHarArgs, StartProxyArgs, WaitArgs } from './typings/proxy';
declare enum Method {
    GET = "GET",
    POST = "POST",
    PUT = "PUT",
    DELETE = "DELETE"
}
export default class BrowserMobProxyAPIClient {
    readonly host: string;
    readonly port: number;
    constructor(host?: string, port?: number);
    protected httpRequest(options: RequestOptions, payload?: string): Promise<string>;
    protected httpRequestWithArgs(path?: string, method?: Method, args?: Record<string, string | number | boolean>): Promise<string>;
    /***
     * Get a list of ports attached to ProxyServer instances managed by ProxyManager
     */
    getProxyList(): Promise<ProxyList>;
    /**
     * Creates a new proxy to run requests off of
     */
    start(args?: StartProxyArgs): Promise<number | null>;
    /**
     * Shuts down the proxy and closes the port.
     */
    stop(port: number): Promise<void>;
    /**
     * Creates a new HAR attached to the proxy
     * returns the HAR content if there was a previous HAR
     */
    startHar(port: number, args?: StartHarArgs): Promise<Har | undefined>;
    /**
     * Starts a new page on the existing HAR
     */
    newHarPage(port: number, args?: NewHarPageArgs): Promise<void>;
    /**
     * Returns the JSON/HAR content representing all the HTTP traffic passed through the proxy
     * provided you have already created the HAR with this.startHar
     */
    getHar(port: number): Promise<Har | undefined>;
    /**
     * Get whitelisted items
     */
    getWhitelist(port: number): Promise<string[]>;
    /**
     * Sets a list of URL patterns to whitelist
     */
    setWhitelist(port: number, args: SetWhitelistArgs): Promise<void>;
    /**
     * Clears all URL patterns from the whitelist
     */
    clearWhitelist(port: number): Promise<void>;
    /**
     * Get blacklisted items
     */
    getBlacklist(port: number): Promise<BlacklistItem[]>;
    /**
     * Set a URL to blacklist
     */
    setBlacklist(port: number, args: SetBlacklistArgs): Promise<void>;
    /**
     * Clears all URL patterns from the blacklist
     */
    clearBlacklist(port: number): Promise<void>;
    /**
     * Limit the bandwidth through the proxy
     */
    setBandwidthLimit(port: number, args?: SetBandwidthLimitArgs): Promise<void>;
    /**
     * Displays the amount of data remaining to be uploaded/downloaded until the limit is reached
     */
    getBandwidthLimit(port: number): Promise<BandwidthLimitsResponse>;
    /**
     * Set and override HTTP Request headers
     * Key is a header name (such as "User-Agent") and value is a value of HTTP header (such as "BrowserMob-Agent")
     */
    setRequestHeaders(port: number, headers: Record<string, string>): Promise<void>;
    /**
     * Overrides normal DNS lookups and remaps the given hosts with the associated IP address
     * Key is a host name (such as "example.com") and value is the IP address (such as "1.2.3.4")
     */
    setHosts(port: number, hosts: Record<string, string>): Promise<void>;
    /**
     * Set username and passwords to be sent for requests on a specific domain that are protected by Basic Authentication
     */
    setBasicAuth(port: number, domain: string, username: string, password: string): Promise<void>;
    /**
     * Wait till all request are being made
     */
    wait(port: number, args: WaitArgs): Promise<unknown>;
    /**
     * Handles different proxy timeouts
     */
    setTimeouts(port: number, timeouts: SetTimeoutsArgs): Promise<void>;
    /**
     * Redirecting URLs
     */
    setRewrite(port: number, args: SetRewriteArgs): Promise<void>;
    /**
     * Removes all URL redirection rules currently in effect
     */
    clearRewrites(port: number): Promise<void>;
    /**
     * Set retry count
     */
    setRetryCount(port: number, args: SetRetryCountArgs): Promise<void>;
    /**
     * Empties the DNS cache
     */
    clearDNSCache(port: number): Promise<void>;
    /**
     * Set request interceptor
     * example:  await proxy.setRequestInterceptor(port, "request.headers().remove('User-Agent'); request.headers().add('User-Agent', 'My-Custom-User-Agent-String 1.0');");
     * @see https://github.com/lightbody/browsermob-proxy#rest-api-interceptors-with-littleproxy
     */
    setRequestInterceptor(port: number, interceptor: string): Promise<void>;
    /**
     * Set response interceptor
     * example: await proxy.setResponseInterceptor(port, "response.headers().remove('User-Agent'); response.headers().add('User-Agent', 'My-Custom-User-Agent-String 1.0');");
     * @see https://github.com/lightbody/browsermob-proxy#rest-api-interceptors-with-littleproxy
     */
    setResponseInterceptor(port: number, interceptor: string): Promise<void>;
}
export {};
